#include <iostream>
#include <i3d/image3d.h>

#include "randWalk.h"

///define whether to produce data for MSD calculation
#define MSD_OUTPUT
const int TimeDeltaMinutes=29; //simulations - cell cycle = 50 frames

#define HISTOGRAM_OUTPUT
//#define INDIVIDUAL_PATHS_OUTPUT
//#define COMBINED_PATHS_OUTPUT


int main(int argc,char **argv)
{
	i3d::Image3d<i3d::GRAY16> pathsI,pathsC,histogram;

#ifdef INDIVIDUAL_PATHS_OUTPUT
	pathsI.MakeRoom(1000,1000,1); //individual paths
#endif
#ifdef COMBINED_PATHS_OUTPUT
	pathsC.MakeRoom(1000,1000,1); //combined paths
	pathsC.GetVoxelData()=0;
#endif
#ifdef HISTOGRAM_OUTPUT
	histogram.MakeRoom(1000,1000,1);
	histogram.GetVoxelData()=0;
#endif

	//init this walk
	RandomWalk<float> rw;
	//rw.UseWrappedCauchyDistribution(0.8); // => r=0.8
	rw.UseWrappedNormalDistribution(0.4); // => r=0.8
	//rw.UseUniformDistribution();
	//My own testing 3-peaks distribution:
	/*
	rw.turningAngleDistribution[-1.0]=0.;
	rw.turningAngleDistribution[-0.8]=1.;
	rw.turningAngleDistribution[-0.6]=0.;
	rw.turningAngleDistribution[-0.2]=0.;
	rw.turningAngleDistribution[ 0.0]=1.;
	rw.turningAngleDistribution[+0.2]=0.;
	rw.turningAngleDistribution[+0.6]=0.;
	rw.turningAngleDistribution[+0.8]=1.;
	rw.turningAngleDistribution[+1.0]=0.;
	*/
	rw.PrepareRandomTurningField();
	float speed=1.;

	//several random walks...
	for (int attemptNo=0; attemptNo < 10000; ++attemptNo)
	{
		//std::cout << "========== " << attemptNo << "==========\n";

		//name this walk
		char attemptName[50];
		sprintf(attemptName,"wALk%06d.tif",attemptNo);
		//if (!(attemptNo % 500)) std::cout << attemptName << "\n";

		//initial position
		float posX=500.f;
		float posY=500.f;
#ifdef INDIVIDUAL_PATHS_OUTPUT
		pathsI.GetVoxelData()=0;
		pathsI.SetVoxel(round(posX),round(posY),0, 100);
#endif
#ifdef COMBINED_PATHS_OUTPUT
		if (attemptNo < 50)
			pathsC.SetVoxel(round(posX),round(posY),0, attemptNo+1);
#endif

#ifdef MSD_OUTPUT
		char fn[100];
		//report...
		sprintf(fn,"T %05d",0*TimeDeltaMinutes);
		if (attemptNo < 300)
		std::cout << "ID " << attemptNo+1 << " " << fn << " POSX " << posX << " POSY " << posY << "\n";
#endif

		//reset initial direction
		// (would rather reset the whole RandomWalk object, but
		//  seeding is not working properly... rapid inits...)
		rw.ResetXYHeading(0.);

		//do the walk...
		for (int step=0; step < 60; ++step)
		{
			//update heading
			float dx,dy;
			rw.DoOneStep2D(dx,dy);

			//update speed...
			//speed += GetRandomGauss(0.f,1.f, &randState);
			
			//update current position
			posX += speed * dx;
			posY += speed * dy;
			
			//draw current position
#ifdef INDIVIDUAL_PATHS_OUTPUT
			pathsI.SetVoxel(round(posX),round(posY),0, 100+5*step);
#endif
#ifdef COMBINED_PATHS_OUTPUT
			if (attemptNo < 50)
				pathsC.SetVoxel(round(posX),round(posY),0, attemptNo+1);
#endif

#ifdef MSD_OUTPUT
			//report...
			sprintf(fn,"T %05d",(step+1)*TimeDeltaMinutes);
			if (attemptNo < 300)
			std::cout << "ID " << attemptNo+1 << " " << fn << " POSX " << posX << " POSY " << posY << "\n";
#endif
		}

#ifdef HISTOGRAM_OUTPUT
		//record the final position into the histogram of end-positions
		if (histogram.Include(round(posX),round(posY),0))
			histogram.SetVoxel(round(posX),round(posY),0,
			           histogram.GetVoxel(round(posX),round(posY),0)+1);
#endif

#ifdef INDIVIDUAL_PATHS_OUTPUT
		if (attemptNo < 50)
			pathsI.SaveImage(attemptName);
#endif
	}

#ifdef COMBINED_PATHS_OUTPUT
	pathsC.SaveImage("wALks_combined.tif");
#endif
#ifdef HISTOGRAM_OUTPUT
	histogram.SaveImage("histogram_endPositions.tif");
#endif

	return(0);
}
